import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="Toolkit-InterFace",
    version="0.3 beta",
    author="XiangQinxi",
    author_email="XiangQinxi@outlook.com",
    description="Tkinter Box",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="MIT",
    url="https://github.com/XiangQinxi/Tki",
    project_urls={
        "Bug Tracker": "https://github.com/XiangQinxi",
    },
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Tcl",
        "License :: OSI Approved :: MIT License",
        "Development Status :: 4 - Beta",
        "License :: Freeware",
        "License :: OSI Approved :: MIT License",
        "Operating System :: MacOS",
        "Operating System :: Microsoft",
        "Operating System :: Unix",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.6",
)